@@ -20,22 +20,24 @@
  */
 
 #include "libavutil/attributes.h"
+
 #include "avcodec.h"
 #include "sinewin.h"
 #include "wma.h"
 #include "wma_common.h"
+#include "wma_freqs.h"
 #include "wmadata.h"
 
 #undef NDEBUG
 #include <assert.h>
 
 /* XXX: use same run/length optimization as mpeg decoders */
-//FIXME maybe split decode / encode or pass flag
+// FIXME maybe split decode / encode or pass flag
 static av_cold void init_coef_vlc(VLC *vlc, uint16_t **prun_table,
                                   float **plevel_table, uint16_t **pint_table,
                                   const CoefVLCTable *vlc_table)
 {
-    int n = vlc_table->n;
+    int n                        = vlc_table->n;
     const uint8_t  *table_bits   = vlc_table->huffbits;
     const uint32_t *table_codes  = vlc_table->huffcodes;
     const uint16_t *levels_table = vlc_table->levels;
@@ -45,20 +47,20 @@ static av_cold void init_coef_vlc(VLC *vlc, uint16_t **prun_table,
 
     init_vlc(vlc, VLCBITS, n, table_bits, 1, 1, table_codes, 4, 4, 0);
 
-    run_table   = av_malloc(n * sizeof(uint16_t));
-    level_table = av_malloc(n * sizeof(uint16_t));
-    flevel_table= av_malloc(n * sizeof(*flevel_table));
-    int_table   = av_malloc(n * sizeof(uint16_t));
-    i = 2;
-    level = 1;
-    k = 0;
+    run_table    = av_malloc_array(n, sizeof(uint16_t));
+    level_table  = av_malloc_array(n, sizeof(uint16_t));
+    flevel_table = av_malloc_array(n, sizeof(*flevel_table));
+    int_table    = av_malloc_array(n, sizeof(uint16_t));
+    i            = 2;
+    level        = 1;
+    k            = 0;
     while (i < n) {
         int_table[k] = i;
-        l = levels_table[k++];
+        l            = levels_table[k++];
         for (j = 0; j < l; j++) {
-            run_table[i]   = j;
-            level_table[i] = level;
-            flevel_table[i]= level;
+            run_table[i]    = j;
+            level_table[i]  = level;
+            flevel_table[i] = level;
             i++;
         }
         level++;
@@ -78,19 +80,17 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
     int sample_rate1;
     int coef_vlc_table;
 
-    if (   avctx->sample_rate <= 0 || avctx->sample_rate > 50000
-        || avctx->channels    <= 0 || avctx->channels    > 2
-        || avctx->bit_rate    <= 0)
+    if (avctx->sample_rate <= 0 || avctx->sample_rate > 50000 ||
+        avctx->channels    <= 0 || avctx->channels    > 2     ||
+        avctx->bit_rate    <= 0)
         return -1;
 
     ff_fmt_convert_init(&s->fmt_conv, avctx);
-    avpriv_float_dsp_init(&s->fdsp, avctx->flags & CODEC_FLAG_BITEXACT);
 
-    if (avctx->codec->id == AV_CODEC_ID_WMAV1) {
+    if (avctx->codec->id == AV_CODEC_ID_WMAV1)
         s->version = 1;
-    } else {
+    else
         s->version = 2;
-    }
 
     /* compute MDCT block size */
     s->frame_len_bits = ff_wma_get_frame_len_bits(avctx->sample_rate,
@@ -109,80 +109,74 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
         if (nb > nb_max)
             nb = nb_max;
         s->nb_block_sizes = nb + 1;
-    } else {
+    } else
         s->nb_block_sizes = 1;
-    }
 
     /* init rate dependent parameters */
     s->use_noise_coding = 1;
-    high_freq = avctx->sample_rate * 0.5;
+    high_freq           = avctx->sample_rate * 0.5;
 
     /* if version 2, then the rates are normalized */
     sample_rate1 = avctx->sample_rate;
     if (s->version == 2) {
-        if (sample_rate1 >= 44100) {
+        if (sample_rate1 >= 44100)
             sample_rate1 = 44100;
-        } else if (sample_rate1 >= 22050) {
+        else if (sample_rate1 >= 22050)
             sample_rate1 = 22050;
-        } else if (sample_rate1 >= 16000) {
+        else if (sample_rate1 >= 16000)
             sample_rate1 = 16000;
-        } else if (sample_rate1 >= 11025) {
+        else if (sample_rate1 >= 11025)
             sample_rate1 = 11025;
-        } else if (sample_rate1 >= 8000) {
+        else if (sample_rate1 >= 8000)
             sample_rate1 = 8000;
-        }
     }
 
-    bps = (float)avctx->bit_rate / (float)(avctx->channels * avctx->sample_rate);
-    s->byte_offset_bits = av_log2((int)(bps * s->frame_len / 8.0 + 0.5)) + 2;
+    bps                 = (float) avctx->bit_rate /
+                          (float) (avctx->channels * avctx->sample_rate);
+    s->byte_offset_bits = av_log2((int) (bps * s->frame_len / 8.0 + 0.5)) + 2;
     if (s->byte_offset_bits + 3 > MIN_CACHE_BITS) {
         av_log(avctx, AV_LOG_ERROR, "byte_offset_bits %d is too large\n", s->byte_offset_bits);
         return AVERROR_PATCHWELCOME;
     }
 
     /* compute high frequency value and choose if noise coding should
-       be activated */
+     * be activated */
     bps1 = bps;
     if (avctx->channels == 2)
         bps1 = bps * 1.6;
     if (sample_rate1 == 44100) {
-        if (bps1 >= 0.61) {
+        if (bps1 >= 0.61)
             s->use_noise_coding = 0;
-        } else {
+        else
             high_freq = high_freq * 0.4;
-        }
     } else if (sample_rate1 == 22050) {
-        if (bps1 >= 1.16) {
+        if (bps1 >= 1.16)
             s->use_noise_coding = 0;
-        } else if (bps1 >= 0.72) {
+        else if (bps1 >= 0.72)
             high_freq = high_freq * 0.7;
-        } else {
+        else
             high_freq = high_freq * 0.6;
-        }
     } else if (sample_rate1 == 16000) {
-        if (bps > 0.5) {
+        if (bps > 0.5)
             high_freq = high_freq * 0.5;
-        } else {
+        else
             high_freq = high_freq * 0.3;
-        }
-    } else if (sample_rate1 == 11025) {
+    } else if (sample_rate1 == 11025)
         high_freq = high_freq * 0.7;
-    } else if (sample_rate1 == 8000) {
-        if (bps <= 0.625) {
+    else if (sample_rate1 == 8000) {
+        if (bps <= 0.625)
             high_freq = high_freq * 0.5;
-        } else if (bps > 0.75) {
+        else if (bps > 0.75)
             s->use_noise_coding = 0;
-        } else {
+        else
             high_freq = high_freq * 0.65;
-        }
     } else {
-        if (bps >= 0.8) {
+        if (bps >= 0.8)
             high_freq = high_freq * 0.75;
-        } else if (bps >= 0.6) {
+        else if (bps >= 0.6)
             high_freq = high_freq * 0.6;
-        } else {
+        else
             high_freq = high_freq * 0.5;
-        }
     }
     av_dlog(s->avctx, "flags2=0x%x\n", flags2);
     av_dlog(s->avctx, "version=%d channels=%d sample_rate=%d bitrate=%d block_align=%d\n",
@@ -198,19 +192,18 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
         int a, b, pos, lpos, k, block_len, i, j, n;
         const uint8_t *table;
 
-        if (s->version == 1) {
+        if (s->version == 1)
             s->coefs_start = 3;
-        } else {
+        else
             s->coefs_start = 0;
-        }
         for (k = 0; k < s->nb_block_sizes; k++) {
             block_len = s->frame_len >> k;
 
             if (s->version == 1) {
                 lpos = 0;
                 for (i = 0; i < 25; i++) {
-                    a = ff_wma_critical_freqs[i];
-                    b = avctx->sample_rate;
+                    a   = ff_wma_critical_freqs[i];
+                    b   = avctx->sample_rate;
                     pos = ((block_len * 2 * a) + (b >> 1)) / b;
                     if (pos > block_len)
                         pos = block_len;
@@ -225,15 +218,14 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
             } else {
                 /* hardcoded tables */
                 table = NULL;
-                a = s->frame_len_bits - BLOCK_MIN_BITS - k;
+                a     = s->frame_len_bits - BLOCK_MIN_BITS - k;
                 if (a < 3) {
-                    if (avctx->sample_rate >= 44100) {
+                    if (avctx->sample_rate >= 44100)
                         table = exponent_band_44100[a];
-                    } else if (avctx->sample_rate >= 32000) {
+                    else if (avctx->sample_rate >= 32000)
                         table = exponent_band_32000[a];
-                    } else if (avctx->sample_rate >= 22050) {
+                    else if (avctx->sample_rate >= 22050)
                         table = exponent_band_22050[a];
-                    }
                 }
                 if (table) {
                     n = *table++;
@@ -241,12 +233,12 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
                         s->exponent_bands[k][i] = table[i];
                     s->exponent_sizes[k] = n;
                 } else {
-                    j = 0;
+                    j    = 0;
                     lpos = 0;
                     for (i = 0; i < 25; i++) {
-                        a = ff_wma_critical_freqs[i];
-                        b = avctx->sample_rate;
-                        pos = ((block_len * 2 * a) + (b << 1)) / (4 * b);
+                        a     = ff_wma_critical_freqs[i];
+                        b     = avctx->sample_rate;
+                        pos   = ((block_len * 2 * a) + (b << 1)) / (4 * b);
                         pos <<= 2;
                         if (pos > block_len)
                             pos = block_len;
@@ -263,16 +255,16 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
             /* max number of coefs */
             s->coefs_end[k] = (s->frame_len - ((s->frame_len * 9) / 100)) >> k;
             /* high freq computation */
-            s->high_band_start[k] = (int)((block_len * 2 * high_freq) /
-                                          avctx->sample_rate + 0.5);
-            n = s->exponent_sizes[k];
-            j = 0;
+            s->high_band_start[k] = (int) ((block_len * 2 * high_freq) /
+                                           avctx->sample_rate + 0.5);
+            n   = s->exponent_sizes[k];
+            j   = 0;
             pos = 0;
             for (i = 0; i < n; i++) {
                 int start, end;
                 start = pos;
-                pos += s->exponent_bands[k][i];
-                end = pos;
+                pos  += s->exponent_bands[k][i];
+                end   = pos;
                 if (start < s->high_band_start[k])
                     start = s->high_band_start[k];
                 if (end > s->coefs_end[k])
@@ -290,7 +282,7 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
             for (j = 0; j < s->exponent_high_sizes[k]; j++)
                 tprintf(s->avctx, " %d", s->exponent_high_bands[k][j]);
             tprintf(s->avctx, "\n");
-#endif
+#endif /* 0 */
         }
     }
 
@@ -306,9 +298,9 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
             tprintf(s->avctx, "\n");
         }
     }
-#endif
+#endif /* TRACE */
 
-    /* init MDCT windows : simple sinus window */
+    /* init MDCT windows : simple sine window */
     for (i = 0; i < s->nb_block_sizes; i++) {
         ff_init_ff_sine_windows(s->frame_len_bits - i);
         s->windows[i] = ff_sine_windows[s->frame_len_bits - i];
@@ -317,13 +309,11 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
     s->reset_block_lengths = 1;
 
     if (s->use_noise_coding) {
-
         /* init the noise generator */
-        if (s->use_exp_vlc) {
+        if (s->use_exp_vlc)
             s->noise_mult = 0.02;
-        } else {
+        else
             s->noise_mult = 0.04;
-        }
 
 #ifdef TRACE
         for (i = 0; i < NOISE_TAB_SIZE; i++)
@@ -333,41 +323,49 @@ av_cold int ff_wma_init(AVCodecContext *avctx, int flags2)
             unsigned int seed;
             float norm;
             seed = 1;
-            norm = (1.0 / (float)(1LL << 31)) * sqrt(3) * s->noise_mult;
+            norm = (1.0 / (float) (1LL << 31)) * sqrt(3) * s->noise_mult;
             for (i = 0; i < NOISE_TAB_SIZE; i++) {
-                seed = seed * 314159 + 1;
-                s->noise_table[i] = (float)((int)seed) * norm;
+                seed              = seed * 314159 + 1;
+                s->noise_table[i] = (float) ((int) seed) * norm;
             }
         }
-#endif
+#endif /* TRACE */
     }
 
+    s->fdsp = avpriv_float_dsp_alloc(avctx->flags & CODEC_FLAG_BITEXACT);
+    if (!s->fdsp)
+        return AVERROR(ENOMEM);
+
     /* choose the VLC tables for the coefficients */
     coef_vlc_table = 2;
     if (avctx->sample_rate >= 32000) {
-        if (bps1 < 0.72) {
+        if (bps1 < 0.72)
             coef_vlc_table = 0;
-        } else if (bps1 < 1.16) {
+        else if (bps1 < 1.16)
             coef_vlc_table = 1;
-        }
     }
-    s->coef_vlcs[0]= &coef_vlcs[coef_vlc_table * 2    ];
-    s->coef_vlcs[1]= &coef_vlcs[coef_vlc_table * 2 + 1];
-    init_coef_vlc(&s->coef_vlc[0], &s->run_table[0], &s->level_table[0], &s->int_table[0],
-                  s->coef_vlcs[0]);
-    init_coef_vlc(&s->coef_vlc[1], &s->run_table[1], &s->level_table[1], &s->int_table[1],
-                  s->coef_vlcs[1]);
+    s->coef_vlcs[0] = &coef_vlcs[coef_vlc_table * 2];
+    s->coef_vlcs[1] = &coef_vlcs[coef_vlc_table * 2 + 1];
+    init_coef_vlc(&s->coef_vlc[0], &s->run_table[0], &s->level_table[0],
+                  &s->int_table[0], s->coef_vlcs[0]);
+    init_coef_vlc(&s->coef_vlc[1], &s->run_table[1], &s->level_table[1],
+                  &s->int_table[1], s->coef_vlcs[1]);
 
     return 0;
 }
 
 int ff_wma_total_gain_to_bits(int total_gain)
 {
-         if (total_gain < 15) return 13;
-    else if (total_gain < 32) return 12;
-    else if (total_gain < 40) return 11;
-    else if (total_gain < 45) return 10;
-    else                      return  9;
+    if (total_gain < 15)
+        return 13;
+    else if (total_gain < 32)
+        return 12;
+    else if (total_gain < 40)
+        return 11;
+    else if (total_gain < 45)
+        return 10;
+    else
+        return  9;
 }
 
 int ff_wma_end(AVCodecContext *avctx)
@@ -378,18 +376,17 @@ int ff_wma_end(AVCodecContext *avctx)
     for (i = 0; i < s->nb_block_sizes; i++)
         ff_mdct_end(&s->mdct_ctx[i]);
 
-    if (s->use_exp_vlc) {
+    if (s->use_exp_vlc)
         ff_free_vlc(&s->exp_vlc);
-    }
-    if (s->use_noise_coding) {
+    if (s->use_noise_coding)
         ff_free_vlc(&s->hgain_vlc);
-    }
     for (i = 0; i < 2; i++) {
         ff_free_vlc(&s->coef_vlc[i]);
         av_freep(&s->run_table[i]);
         av_freep(&s->level_table[i]);
         av_freep(&s->int_table[i]);
     }
+    av_freep(&s->fdsp);
 
     return 0;
 }
@@ -399,7 +396,7 @@ int ff_wma_end(AVCodecContext *avctx)
  * @param gb GetBitContext
  * @return the decoded coefficient
  */
-unsigned int ff_wma_get_large_val(GetBitContext* gb)
+unsigned int ff_wma_get_large_val(GetBitContext *gb)
 {
     /** consumes up to 34 bits */
     int n_bits = 8;
@@ -408,9 +405,8 @@ unsigned int ff_wma_get_large_val(GetBitContext* gb)
         n_bits += 8;
         if (get_bits1(gb)) {
             n_bits += 8;
-            if (get_bits1(gb)) {
+            if (get_bits1(gb))
                 n_bits += 7;
-            }
         }
     }
     return get_bits_long(gb, n_bits);
@@ -432,24 +428,24 @@ unsigned int ff_wma_get_large_val(GetBitContext* gb)
  * @param coef_nb_bits number of bits for escaped level codes
  * @return 0 on success, -1 otherwise
  */
-int ff_wma_run_level_decode(AVCodecContext* avctx, GetBitContext* gb,
-                            VLC *vlc,
-                            const float *level_table, const uint16_t *run_table,
-                            int version, WMACoef *ptr, int offset,
-                            int num_coefs, int block_len, int frame_len_bits,
+int ff_wma_run_level_decode(AVCodecContext *avctx, GetBitContext *gb,
+                            VLC *vlc, const float *level_table,
+                            const uint16_t *run_table, int version,
+                            WMACoef *ptr, int offset, int num_coefs,
+                            int block_len, int frame_len_bits,
                             int coef_nb_bits)
 {
     int code, level, sign;
-    const uint32_t *ilvl = (const uint32_t*)level_table;
-    uint32_t *iptr = (uint32_t*)ptr;
+    const uint32_t *ilvl = (const uint32_t *) level_table;
+    uint32_t *iptr = (uint32_t *) ptr;
     const unsigned int coef_mask = block_len - 1;
     for (; offset < num_coefs; offset++) {
         code = get_vlc2(gb, vlc->table, VLCBITS, VLCMAX);
         if (code > 1) {
             /** normal code */
-            offset += run_table[code];
-            sign = get_bits1(gb) - 1;
-            iptr[offset & coef_mask] = ilvl[code] ^ sign<<31;
+            offset                  += run_table[code];
+            sign                     = get_bits1(gb) - 1;
+            iptr[offset & coef_mask] = ilvl[code] ^ sign << 31;
         } else if (code == 1) {
             /** EOB */
             break;
@@ -458,7 +454,7 @@ int ff_wma_run_level_decode(AVCodecContext* avctx, GetBitContext* gb,
             if (!version) {
                 level = get_bits(gb, coef_nb_bits);
                 /** NOTE: this is rather suboptimal. reading
-                    block_len_bits would be better */
+                 *  block_len_bits would be better */
                 offset += get_bits(gb, frame_len_bits);
             } else {
                 level = ff_wma_get_large_val(gb);
@@ -466,8 +462,8 @@ int ff_wma_run_level_decode(AVCodecContext* avctx, GetBitContext* gb,
                 if (get_bits1(gb)) {
                     if (get_bits1(gb)) {
                         if (get_bits1(gb)) {
-                            av_log(avctx,AV_LOG_ERROR,
-                                "broken escape sequence\n");
+                            av_log(avctx, AV_LOG_ERROR,
+                                   "broken escape sequence\n");
                             return -1;
                         } else
                             offset += get_bits(gb, frame_len_bits) + 4;
@@ -475,8 +471,8 @@ int ff_wma_run_level_decode(AVCodecContext* avctx, GetBitContext* gb,
                         offset += get_bits(gb, 2) + 1;
                 }
             }
-            sign = get_bits1(gb) - 1;
-            ptr[offset & coef_mask] = (level^sign) - sign;
+            sign                    = get_bits1(gb) - 1;
+            ptr[offset & coef_mask] = (level ^ sign) - sign;
         }
     }
     /** NOTE: EOB can be omitted */
