@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include <inttypes.h>
+
 #include "libavutil/channel_layout.h"
 #include "libavutil/common.h"
 #include "avformat.h"
@@ -116,12 +118,10 @@ static int get_sindex(AVFormatContext *s, int id, int format) {
             st->codec->codec_id = AV_CODEC_ID_MJPEG;
             break;
         case 13:
-        case 15:
-            st->codec->codec_type = AVMEDIA_TYPE_VIDEO;
-            st->codec->codec_id = AV_CODEC_ID_DVVIDEO;
-            break;
         case 14:
+        case 15:
         case 16:
+        case 25:
             st->codec->codec_type = AVMEDIA_TYPE_VIDEO;
             st->codec->codec_id = AV_CODEC_ID_DVVIDEO;
             break;
@@ -165,6 +165,12 @@ static int get_sindex(AVFormatContext *s, int id, int format) {
             st->codec->channel_layout = AV_CH_LAYOUT_STEREO;
             st->codec->sample_rate = 48000;
             break;
+        case 26: /* AVCi50 / AVCi100 (AVC Intra) */
+        case 29: /* AVCHD */
+            st->codec->codec_type = AVMEDIA_TYPE_VIDEO;
+            st->codec->codec_id = AV_CODEC_ID_H264;
+            st->need_parsing = AVSTREAM_PARSE_HEADERS;
+            break;
         // timecode tracks:
         case 7:
         case 8:
@@ -172,6 +178,10 @@ static int get_sindex(AVFormatContext *s, int id, int format) {
             st->codec->codec_type = AVMEDIA_TYPE_DATA;
             st->codec->codec_id = AV_CODEC_ID_NONE;
             break;
+        case 30:
+            st->codec->codec_type = AVMEDIA_TYPE_VIDEO;
+            st->codec->codec_id = AV_CODEC_ID_DNXHD;
+            break;
         default:
             st->codec->codec_type = AVMEDIA_TYPE_UNKNOWN;
             st->codec->codec_id = AV_CODEC_ID_NONE;
@@ -285,7 +295,9 @@ static void gxf_read_index(AVFormatContext *s, int pkt_len) {
     }
     st = s->streams[0];
     if (map_cnt > 1000) {
-        av_log(s, AV_LOG_ERROR, "too many index entries %u (%x)\n", map_cnt, map_cnt);
+        av_log(s, AV_LOG_ERROR,
+               "too many index entries %"PRIu32" (%"PRIx32")\n",
+               map_cnt, map_cnt);
         map_cnt = 1000;
     }
     if (pkt_len < 4 * map_cnt) {
@@ -430,7 +442,7 @@ static int gxf_header(AVFormatContext *s) {
 
 #define READ_ONE() \
     { \
-        if (!max_interval-- || url_feof(pb)) \
+        if (!max_interval-- || avio_feof(pb)) \
             goto out; \
         tmp = tmp << 8 | avio_r8(pb); \
     }
@@ -492,7 +504,7 @@ static int gxf_packet(AVFormatContext *s, AVPacket *pkt) {
         int field_nr, field_info, skip = 0;
         int stream_index;
         if (!parse_packet_header(pb, &pkt_type, &pkt_len)) {
-            if (!url_feof(pb))
+            if (!avio_feof(pb))
                 av_log(s, AV_LOG_ERROR, "sync lost\n");
             return -1;
         }
@@ -548,7 +560,7 @@ static int gxf_packet(AVFormatContext *s, AVPacket *pkt) {
 }
 
 static int gxf_seek(AVFormatContext *s, int stream_index, int64_t timestamp, int flags) {
-    int64_t res = 0;
+    int res = 0;
     uint64_t pos;
     uint64_t maxlen = 100 * 1024 * 1024;
     AVStream *st = s->streams[0];
